/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include <Xm/ToggleB.h>
#include "xmap.h"
#include "mode.h"
#include "debug.h"

extern Widget draw_w;
extern Widget toplevel;
extern struct mapdata mapdata;

static Widget dialog = NULL;
static Widget bul;
static Widget ok;
static Widget cancel;
static Widget help;
static Widget b1, b2;

static int answer = ANS_NONE;

static void ok_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_OK;
}

static void cancel_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_CANCEL;
}


static void button1_cb(Widget dialog, XtPointer cd, XmToggleButtonCallbackStruct *cb)
{
	if (cb->set) {
		XmToggleButtonSetState(b2, False, False);
		mapdata.calstate = 0;
		ch_mode(CALIBRATE,START);
	}
	else {
		ch_mode(CALIBRATE,STOP);
	}
}

static void button2_cb(Widget dialog, XtPointer cd, XmToggleButtonCallbackStruct *cb)
{
	if (cb->set) {
		XmToggleButtonSetState(b1, False, False);
		mapdata.calstate = 1;
		ch_mode(CALIBRATE,START);
	}
	else {
		ch_mode(CALIBRATE,STOP);
	}
}


void cal_dialog(Widget w)
{
    Pixel foreground, background, select;
    XtAppContext app = XtWidgetToApplicationContext(toplevel);
    XmString s;

    if (mapdata.type == VECTOR) {
	error("Vector maps do not need to be calibrated!\n", 0);
	return;
    }

    if (!dialog) {

	foreground = get_pixel(draw_w, "Black");
	background = get_pixel(draw_w, "Snow");
	select     = get_pixel(draw_w, "Red");
	dialog = XtVaCreatePopupShell("latlon",
			   xmDialogShellWidgetClass, GetTopShell(draw_w),
				      XmNdeleteResponse, XmUNMAP,
				      XmNresizePolicy, XmRESIZE_NONE,
				      XmNnoResize, True,
				      XmNwidth, 290,
				      XmNminWidth, 290,
				      XmNmaxWidth, 290,
				      XmNheight, 130,
				      XmNminHeight, 130,
				      XmNmaxHeight, 130,
				      NULL);

	bul = XtVaCreateWidget("bul",
			       xmBulletinBoardWidgetClass, dialog,
			       XmNforeground, foreground,
			       XmNautoUnmanage, False,
			       XmNdialogStyle, XmDIALOG_WORK_AREA,
			       NULL);
	XtAddCallback(bul, XmNunmapCallback, (void *) cancel_cb, NULL);

	s = XmStringCreateSimple(" Select Point #1");
	b1 = XtVaCreateManagedWidget("t1",
			xmToggleButtonWidgetClass, bul,
			XmNhighlightThickness, 0,
			XmNvisibleWhenOff, True,
			XmNindicatorType, XmONE_OF_MANY,
			XmNselectColor, select,
			XmNlabelString, s,
			XmNx, 40,
			XmNy, 33,
			NULL);
	XmStringFree(s);
	XtAddCallback(b1, XmNvalueChangedCallback, (XtCallbackProc)button1_cb, NULL);

	s = XmStringCreateSimple(" Select Point #2");
	b2 = XtVaCreateManagedWidget("t2",
			xmToggleButtonWidgetClass, bul,
			XmNhighlightThickness, 0,
			XmNvisibleWhenOff, True,
			XmNindicatorType, XmONE_OF_MANY,
			XmNselectColor, select,
			XmNlabelString, s,
			XmNx, 40,
			XmNy, 68,
			NULL);
	XmStringFree(s);
	XtAddCallback(b2, XmNvalueChangedCallback, (XtCallbackProc)button2_cb, NULL);
	
	


	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, bul,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 0,
					 XmNwidth, 60,
					 XmNx, 200,
					 XmNy, 68,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, cancel_cb, NULL);

	ok = XtVaCreateManagedWidget("OK",
				     xmPushButtonWidgetClass, bul,
				     XmNforeground, foreground,
				     XmNhighlightThickness, 0,
				     XmNwidth, 60,
				     XmNx, 200,
				     XmNy, 33,
				     NULL);
	XtAddCallback(ok, XmNactivateCallback, ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, bul,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 0,
				       XmNwidth, 60,
				       XmNx, 200,
				       XmNy, 103,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, show_help_cb, "calibratemaps.html");
    }

    XmToggleButtonSetState(b1, False, False);
    XmToggleButtonSetState(b2, False, False);

    XtManageChild(bul);

    XtVaSetValues(dialog,
		  XmNtitle, "Calibrate Map",
		  NULL);
    XtManageChild(dialog);
    XtPopup(dialog, XtGrabNone);

    answer = ANS_NONE;

    while (answer == ANS_NONE)
	XtAppProcessEvent(app, XtIMAll);

    ch_mode(CALIBRATE,STOP);
    XtPopdown(dialog);

    if (answer == ANS_OK) {
	if (do_calibrate())
	    update_ini(&mapdata);
	else {
	    error("The coordinates you entered are invalid!\n"
		  "  Please read the help information and\n"
		  "           try again.\n",0);
	}
    }
    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(toplevel);

    return;
}

